typedef signed   char  	    SChar;
typedef unsigned char  	    UChar;
typedef signed   short 	    SShort;
typedef unsigned short 	    UShort;
typedef signed   int   	    SInt;
typedef unsigned int   	    UInt;
typedef signed   long  	    SLong;
typedef unsigned long  	    ULong;
typedef signed   long long  SLongLong;
typedef unsigned long long  ULongLong;
