# -*- coding: utf-8 -*-
# cython: language_level=3

cdef Fα1()
cdef class Γναμε2:
    cdef public int α
    cdef boring_cdef(self)
    cdef εxciting_cdef(self)
    cpdef boring_cpdef(self)
    cpdef εxciting_cpdef(self)
    cpdef cpdef_with_exciting_arg(self, ααα)
